/*
           Copyright Devin DeLong 2006 - 2007.
  Distributed under the Boost Software License, Version 1.0.
     (See accompanying file LICENSE_1_0.txt or copy at
           http://www.boost.org/LICENSE_1_0.txt)
 */


#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

//bzfs
#include "bzfsAPI.h"

//local
#include "Mine.h"

// Boost
#include <boost/array.hpp>


Mine::Mine() : playerID_(-1), duration_(0), mineType_(eNullMine)
{
  pos_.assign(0);
}


Mine::Mine(int player, const boost::array<float, 3>& position, eMineType mineType) : 
  playerID_(player),
  pos_(position), 
  duration_(0),
  mineType_(mineType)
{ 
  // Nothing
}


SRMine::SRMine(int player, const boost::array<float, 3>& position): 
  Mine(player, position, eRegularSphereMine)
{ 
  duration_ = static_cast<float>(bz_getBZDBDouble("_shockOutRadius") / 
                                (bz_getBZDBDouble("_rFireAdVel") * 
                                 bz_getBZDBDouble("_shotSpeed")) * 2);
}




BFMine::BFMine(int player, const boost::array<float, 3>& position): 
  Mine(player, position, eBadFlagMine)
{ 
  // Nothing to do
}


SBMine::SBMine(int player, const boost::array<float, 3>& position): 
Mine(player, position, eBadFlagMine)
{ 
  duration_ = static_cast<float>(bz_getBZDBDouble("_shockOutRadius") / 
                                 bz_getBZDBDouble("_shotSpeed") * 2);
}


CRMine::CRMine(int player, const boost::array<float, 3>& position): 
  Mine(player, position, eRegularCylinderMine)
{ 
  duration_ = static_cast<float>(bz_getBZDBDouble("_shockOutRadius") / 
                                 (bz_getBZDBDouble("_rFireAdVel") * 
                                  bz_getBZDBDouble("_shotSpeed")) * 2);
}

SWMine::SWMine(int player, const boost::array<float, 3>& position) : 
  Mine(player, position, eShockWaveMine)
{ 
  // Nothing
}



void SWMine::fire()
{
  bz_fireWorldWep("SW", 2.0f, BZ_SERVER, pos_.c_array(), 0, 0, kMineShotIDMin+26, 0.0f);
}


void SRMine::fire()
{
  float dt = 0.25f;  
  //straight up
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), M_PI/2, 0, kMineShotIDMin, dt);  
  //straight down
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 3*M_PI/2, 0, kMineShotIDMin+1, dt);
  
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 0, kMineShotIDMin+2, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, M_PI/4, kMineShotIDMin+3, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, M_PI/2, kMineShotIDMin+4, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 3*M_PI/4, kMineShotIDMin+5, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, M_PI, kMineShotIDMin+6, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 5*M_PI/4, kMineShotIDMin+7, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 3*M_PI/2, kMineShotIDMin+8, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 7*M_PI/4, kMineShotIDMin+9, dt);
  
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, 0, kMineShotIDMin+10, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, M_PI/4, kMineShotIDMin+11, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, M_PI/2, kMineShotIDMin+12, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, 3*M_PI/4, kMineShotIDMin+13, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, M_PI, kMineShotIDMin+14, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, 5*M_PI/4, kMineShotIDMin+15, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, 3*M_PI/2, kMineShotIDMin+16, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, 7*M_PI/4, kMineShotIDMin+17, dt);
  
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, 0, kMineShotIDMin+18, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, M_PI/4, kMineShotIDMin+19, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, M_PI/2, kMineShotIDMin+20, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, 3*M_PI/4, kMineShotIDMin+21, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, M_PI, kMineShotIDMin+22, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, 5*M_PI/4, kMineShotIDMin+23, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, 3*M_PI/2, kMineShotIDMin+24, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, 7*M_PI/4, kMineShotIDMin+25, dt);
}


void SBMine::fire()
{
  float dt = 0.25f;  
  //straight up
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), M_PI/2, 0, kMineShotIDMin, dt);  
  //straight down
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 3*M_PI/2, 0, kMineShotIDMin+1, dt);
  
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 0, 0, kMineShotIDMin+2, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 0, M_PI/4, kMineShotIDMin+3, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 0, M_PI/2, kMineShotIDMin+4, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 0, 3*M_PI/4, kMineShotIDMin+5, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 0, M_PI, kMineShotIDMin+6, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 0, 5*M_PI/4, kMineShotIDMin+7, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 0, 3*M_PI/2, kMineShotIDMin+8, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 0, 7*M_PI/4, kMineShotIDMin+9, dt);
  
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, 0, kMineShotIDMin+10, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, M_PI/4, kMineShotIDMin+11, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, M_PI/2, kMineShotIDMin+12, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, 3*M_PI/4, kMineShotIDMin+13, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, M_PI, kMineShotIDMin+14, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, 5*M_PI/4, kMineShotIDMin+15, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, 3*M_PI/2, kMineShotIDMin+16, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), M_PI/4, 7*M_PI/4, kMineShotIDMin+17, dt);
  
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, 0, kMineShotIDMin+18, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, M_PI/4, kMineShotIDMin+19, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, M_PI/2, kMineShotIDMin+20, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, 3*M_PI/4, kMineShotIDMin+21, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, M_PI, kMineShotIDMin+22, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, 5*M_PI/4, kMineShotIDMin+23, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, 3*M_PI/2, kMineShotIDMin+24, dt);
  bz_fireWorldWep("SB", duration_, BZ_SERVER, pos_.c_array(), 7*M_PI/4, 7*M_PI/4, kMineShotIDMin+25, dt);
}



void CRMine::fire()  
{
  float dt = 0.25f;  
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 0, kMineShotIDMin+27, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, M_PI/4, kMineShotIDMin+28, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, M_PI/2, kMineShotIDMin+29, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 3*M_PI/4, kMineShotIDMin+30, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, M_PI, kMineShotIDMin+31, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 5*M_PI/4, kMineShotIDMin+32, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 3*M_PI/2, kMineShotIDMin+33, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 7*M_PI/4, kMineShotIDMin+34, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, M_PI/6, kMineShotIDMin+35, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, M_PI/3, kMineShotIDMin+36, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 2*M_PI/3, kMineShotIDMin+37, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 5*M_PI/6, kMineShotIDMin+38, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 7*M_PI/6, kMineShotIDMin+39, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 4*M_PI/3, kMineShotIDMin+40, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 5*M_PI/3, kMineShotIDMin+41, dt);
  bz_fireWorldWep("F", duration_, BZ_SERVER, pos_.c_array(), 0, 11*M_PI/6, kMineShotIDMin+42, dt);
}

