/*
           Copyright Devin DeLong 2006 - 2007.
  Distributed under the Boost Software License, Version 1.0.
     (See accompanying file LICENSE_1_0.txt or copy at
           http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef Mine_H
#define Mine_H

// bzfs
#include "bzfsAPI.h"

// boost
#include <boost/array.hpp>

#define kMineShotIDMin 100
#define kMineShotIDMax 142



enum eMineType {
  eShockWaveMine, 
  eRegularSphereMine,
  eRegularCylinderMine,
  eSuperBulletMine,
  eBadFlagMine,
  eNullMine
};



class Mine {       
  
 virtual Mine* do_clone() const = 0;
  
public:
  
  // Contructor
  Mine();  
  
  // Constructor
  Mine(int player, const boost::array<float, 3>& position, eMineType mineType);
  
  // Virtual!
  virtual ~Mine() { }
    
  // Blows up the mine
  virtual void fire() = 0;
  
  // Get the playerID
  int playerID() const { return playerID_; }
  
  // Get the position
  const boost::array<float, 3>& position() const { return pos_; }
  
  // Gget the mine type
  eMineType type() const { return mineType_; }
  
  Mine* clone() const
  {
    return do_clone();
  }
  
protected:
    
  // The player who left the mine
  int playerID_;
  
  // The position of the mine.
  boost::array<float, 3> pos_;
  
  // Controls the blast radius.
  float duration_;
  
  // The Mine Type
  eMineType mineType_;
};


inline Mine* new_clone(const Mine& a)
{
  return a.clone();
}


inline bool operator==(const Mine& mine, int ID)
{
  return mine.playerID() == ID;
}


class SWMine : public Mine {

public: 
  
  SWMine() {}
  SWMine(int player, const boost::array<float, 3>& position);  

  virtual void fire();
  
private:
  
  virtual Mine* do_clone() const
  {
    return new SWMine(this->playerID_, this->pos_);
  }

};



class SRMine : public Mine {

public:

  SRMine() {}
  SRMine(int player, const boost::array<float, 3>& position);  
  
  virtual void fire();
  
  private:
  
  virtual Mine* do_clone() const
  {
    return new SRMine(this->playerID_, this->pos_);
  }

};



class CRMine : public Mine {

public:

  CRMine() {}
  CRMine(int player, const boost::array<float, 3>& position);  
  
  virtual void fire();
  
  private:
  
  virtual Mine* do_clone() const
  {
    return new CRMine(this->playerID_, this->pos_);
  }

};


class SBMine : public Mine {
  
public:
  
  SBMine() {}
  SBMine(int player, const boost::array<float, 3>& position);  
  
  virtual void fire();
  
  private:
  
  virtual Mine* do_clone() const
  {
    return new SBMine(this->playerID_, this->pos_);
  }
  
};


class BFMine : public Mine {
  
public:
  
  BFMine() {}
  BFMine(int player, const boost::array<float, 3>& position);  
  
  // What to do?
  virtual void fire() {};
  
private:
  
  virtual Mine* do_clone() const
  {
    return new BFMine(this->playerID_, this->pos_);
  }
    
};



#endif /* Mine_H */














