/*
           Copyright Devin DeLong 2006 - 2007.
  Distributed under the Boost Software License, Version 1.0.
     (See accompanying file LICENSE_1_0.txt or copy at
           http://www.boost.org/LICENSE_1_0.txt)
 */

// BZFS
#include "bzfsAPI.h"

// Local
#include "utility.h"
#include "Global.h"

// STL
#include <cmath>
#include <string>

static std::string badFlags[12] = 
  {"BY", "RC", "FO", "LT", "RT", "O", "JM", "CB", "CB", "B", "TR", "WA"};

void giveBadFlag(int victimID)
{
  
  for(std::size_t i = 0; i < 12; ++i)
    if(bz_givePlayerFlag(victimID, badFlags[i].c_str(), true))
      break;  
}


std::string toSTLString(const bzApiString& string)
{
  return std::string(string.c_str());
}


void toSTLVector(std::vector<std::string>& newVec,
                 bzAPIStringList& oldVec)
{
  for(std::size_t i = 0; i < oldVec.size(); ++i)
    newVec.push_back(std::string(oldVec[i].c_str()));
}


float distance3f(const Mine& p1, float const* p2)
{
  return std::sqrt((p1.position()[0] - p2[0]) * (p1.position()[0] - p2[0]) +
                   (p1.position()[1] - p2[1]) * (p1.position()[1] - p2[1]) +
                   (p1.position()[2] - p2[2]) * (p1.position()[2] - p2[2]));
}


float distance3f(const boost::array<float, 3>& p1, const boost::array<float, 3>& p2)
{
  return std::sqrt((p1[0] - p2[0]) * (p1[0] - p2[0]) +
                   (p1[1] - p2[1]) * (p1[1] - p2[1]) +
                   (p1[2] - p2[2]) * (p1[2] - p2[2]));
}


float distance3f(float const* p1, float const* p2)
{
  return std::sqrt((p1[0] - p2[0]) * (p1[0] - p2[0]) +
                   (p1[1] - p2[1]) * (p1[1] - p2[1]) +
                   (p1[2] - p2[2]) * (p1[2] - p2[2]));
}


float distance2f(const Mine& p1, float const* p2)
{
  return std::sqrt((p1.position()[0] - p2[0]) * (p1.position()[0] - p2[0]) +
                   (p1.position()[1] - p2[1]) * (p1.position()[1] - p2[1]));
}


float distance2f(const boost::array<float, 2>& p1, const boost::array<float, 2>& p2)
{
  return std::sqrt((p1[0] - p2[0]) * (p1[0] - p2[0]) +
                   (p1[1] - p2[1]) * (p1[1] - p2[1]));
}


float distance2f(float const* p1, float const* p2)
{
  return std::sqrt((p1[0] - p2[0]) * (p1[0] - p2[0]) +
                   (p1[1] - p2[1]) * (p1[1] - p2[1]));
}


void getPlayerIndexList(std::vector<int>& newList)
{
  bzAPIIntList *playerList = bz_newIntList();
  
  if(playerList) {
    bz_getPlayerIndexList(playerList);
    
    for(std::size_t i = 0; i < playerList->size(); ++i)
      newList.push_back((*playerList)[i]);
    
    bz_deleteIntList(playerList);
  }
}


void getTeamBases(std::vector<TeamBase>& teamBases)
{
  std::size_t numFlags = bz_getNumFlags();
  std::size_t i = 0;
  
  while(i < numFlags) {
    std::string temp = toSTLString(bz_getName(i));
    float pos[3];
    
    if(temp == "R*") {
      bz_resetFlag(i);
      bz_getFlagPosition(i, pos);
      teamBases.push_back(TeamBase(pos, eRedTeam));
    } else if(temp == "B*") {
      bz_resetFlag(i);
      bz_getFlagPosition(i, pos);
      teamBases.push_back(TeamBase(pos, eBlueTeam));
    }else if(temp == "G*") {
      bz_resetFlag(i);
      bz_getFlagPosition(i, pos);
      teamBases.push_back(TeamBase(pos, eGreenTeam));
    } else if(temp == "P*") {
      bz_resetFlag(i);
      bz_getFlagPosition(i, pos);
      teamBases.push_back(TeamBase(pos, ePurpleTeam));
    }
    
    ++i;
  }
}



void sendKilledMsg(int victimID, int killerID)
{
  bz_PlayerRecord *victim = bz_getPlayerByIndex(victimID);
  if(!victim) return;
  
  bz_PlayerRecord *killer = bz_getPlayerByIndex(killerID);
  if(!killer) { bz_freePlayerRecord(victim); return; }
  
  static boost::format victimFormat = 
    boost::format("You were destroyed by %1%'s mine");
  
  static boost::format killerFormat = 
    boost::format("%1% was destroyed by your mine");
  
  if(killer->playerID == victim->playerID) {
    bz_sendTextMessage(killerID, killerID, "You were destroyed by your own mine.");
  } else {
    std::string msgV = boost::str(victimFormat % killer->callsign.c_str());
    std::string msgK = boost::str(killerFormat % victim->callsign.c_str());
    
    /** Send the message to the sender so that it appears
     in white text in the victim's console */
    bz_sendTextMessage(victimID, victimID, msgV.c_str());
    bz_sendTextMessage(killerID, killerID, msgK.c_str());
  }
  
  bz_freePlayerRecord(killer);
  bz_freePlayerRecord(victim);
}















